/******************************************************************************/
/* This software is Copyright (C) 1998 by Rafal Wierzbicki. I accept no         */
/* responsibility for anything this software may or may not do to your system   */
/* - you use it completely at your own risk. This software comes under the GPL*/
/* licence. See the COPYING file for full legal details.                        */
/******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <stdlib.h>
#include <dirent.h>
#include <sys/types.h>
#include <gtk/gtk.h>
#include "Pictorial.xpm"
#define CENTER 1
#define TILE 2
#define SCALE 3

/*Prototypes*/

    	/*err_dialog.c*/
void error_dialog (char *error_message);
    	/*wallp_root.c*/
int put_image_on_root (char *imageFile, int STYLE);
    	/*local*/
static void file_selection ();
static gint list_show ();
int STYLE;
static int AUTO_QUIT=0;
static int AUTO_START=0;

const char *data_key="";
static unsigned int no_of_list_items = 0;

static GtkWidget *list;
static GtkEntry *entry;

static char save_file[128];

static void 
f_save_file ()
{
   if (!sprintf (save_file, "%s%s", getenv("HOME"), "/.wallp_save"))
    error_dialog ("Could not save the image list! ~/.wallp_save\n");
}

static unsigned int
random_image ()
{
   unsigned int j;
   
   j=0+(int) ( (float) no_of_list_items * rand()/(RAND_MAX+1.0));
   return j;
}

static int timer = 0;

static void
start_idle (GtkWidget *w, GtkEntry *entry)
{
   if (!timer){
      timer = gtk_timeout_add (100, (GtkFunction) list_show, entry);
   }
}

void static
stop_idle ()
{
   if (timer){
      gtk_timeout_remove (timer);
      timer = 0;
   }
}

static void
list_add (char *buffer)
{
   GtkWidget *list_item, *label;
   gchar *string;
   
   no_of_list_items = no_of_list_items + 1;
   label = gtk_label_new (buffer);
   list_item = gtk_list_item_new ();
   gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
   gtk_container_add (GTK_CONTAINER (list_item), label);
   gtk_widget_show (label);
   gtk_container_add (GTK_CONTAINER (list), list_item);
   gtk_widget_show (list_item);
   gtk_label_get (GTK_LABEL (label), &string);
   gtk_object_set_data (GTK_OBJECT (list_item), data_key,
			string);
}

static void
list_remove (GtkWidget *widget,
             GtkWidget *list)
{
  GList *tmp_list;
  GList *clear_list;

  tmp_list = GTK_LIST (list)->selection;
  clear_list = NULL;

  while (tmp_list)
    {
      clear_list = g_list_prepend (clear_list, tmp_list->data);
      tmp_list = tmp_list->next;
    }
   
   clear_list = g_list_reverse (clear_list);
   
   gtk_list_remove_items (GTK_LIST (list), clear_list);
   if (no_of_list_items > 0)
     no_of_list_items = no_of_list_items - 1;
   else 
     no_of_list_items = 0;
   
   g_list_free (clear_list);        
}

int
list_items ()
{
   GList *tmp_list;
   int items=0;
   
   tmp_list = GTK_LIST (list)->children;
   while (tmp_list){
      tmp_list=tmp_list->next;
      items = items + 1;
   }
   return items;
}

static void
remove_all ()
{
    gtk_list_remove_items (GTK_LIST (list), GTK_LIST(list)->children);
    no_of_list_items = 0;
}

static void
dump_list_to_file ()
{
   GList *tmp_list;
   GtkObject *list_item;
   char *item_data;
   int i;
   FILE *fp;

   if (!(fp = fopen (save_file, "w"))){
    error_dialog ("Could not open the save file ~/.wallp_save\n");
    return;
    }
   
   fprintf (fp, "%s\n", gtk_entry_get_text (entry));
   tmp_list = GTK_LIST (list)->children;
   for (i = 0; i < no_of_list_items; i++)
     {
	gtk_list_select_item (GTK_LIST (list), i);
	tmp_list = GTK_LIST (list)->selection;
	list_item = GTK_OBJECT (tmp_list->data);
	item_data = gtk_object_get_data (list_item, data_key);
	fprintf (fp, "%s\n", item_data);
      }   
   fclose (fp);
}

static void
read_list_from_file ()
{
   FILE *fp;
   char tmp[128];
      
   fp = fopen (save_file, "r");
   if (fp)
     {	
     	fgets (tmp, 127, fp);
	tmp[strlen(tmp)-1] = '\0';
	gtk_entry_set_text (entry, tmp);
	while (1)
	  {	     
	     fgets (tmp, 127, fp);
	     tmp[strlen(tmp)-1] = '\0';
	     if (!feof(fp))
	     	 list_add (tmp);
	     else
	       break;
	  }
	fclose (fp);
     }
}


static int calls = 0;

static
gint stop_all ()
{
    calls = 0;
    stop_idle ();
    return FALSE;
}

static
gint list_show ()
{
   static GList *tmp_list;
   GtkObject *list_item;
   char *item_data;
   static int sleep_time;
   static time_t time_start;
   time_t time_now;

   time_now = time (NULL);
   
   if (calls == 0){
      calls = 1;
      gtk_list_select_item (GTK_LIST (list), (gint) random_image ());
      tmp_list = GTK_LIST (list)->selection;
      list_item = GTK_OBJECT (tmp_list->data);
      item_data = gtk_object_get_data (list_item, data_key);
      dump_list_to_file();   
      if (put_image_on_root (item_data, STYLE)){
        puts ("fucked");
      	list_remove (GTK_WIDGET (list_item), list);
	}
      time_start = time (NULL);
   }      

   if (entry)
     {
	sleep_time = atoi (gtk_entry_get_text (entry)) * 60;
	if (sleep_time == 0){
    	    stop_all ();
	    error_dialog ("Sleep time is 0 minutes
Enter new time and press OK
in the main window");
	    return FALSE;
	}
     }
   else
     {
     	stop_all ();
	return FALSE;
     }
   
      
   if ((time_now - time_start) >= sleep_time )
     {      
	
	if (no_of_list_items == 0)
	  {
	     stop_all ();
	     return FALSE;
	  }
	gtk_list_select_item (GTK_LIST (list), (gint) random_image ());
	tmp_list = GTK_LIST (list)->selection;
	list_item = GTK_OBJECT (tmp_list->data);
	item_data = gtk_object_get_data (list_item, data_key);
	if (put_image_on_root (item_data, STYLE)){
	    puts ("fucked");
	    list_remove (GTK_WIDGET (list_item), list);
	}
	time_start = time (NULL);
     }
   return TRUE;
}

static void
close_button (GtkWidget *w, GtkWidget *window)
{
   dump_list_to_file ();
   stop_all ();
   gtk_widget_destroy (window);
}

static void 
create_list ()
{
  static GtkWidget *window = NULL;
   GtkWidget *vbox1, *vbox2, *vbox3;
   GtkWidget *hbox1, *hbox2;
   GtkWidget *scrolled_win;
   GtkWidget *button;
   GtkWidget *separator;
   GtkWidget *label;
   GtkWidget *frame1, *frame2;
   GtkWidget *pixmapwid;
   GdkPixmap *pixmap;
   GdkBitmap *mask;
   GtkStyle *style;
   
  if (!window)
    {
       window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
       gtk_window_set_title (GTK_WINDOW (window), "WallP");
       gtk_widget_set_usize (window, 530, 275);
       gtk_widget_set_uposition (window, 20, 20);
       gtk_widget_realize (window);
       
       gtk_signal_connect (GTK_OBJECT (window), "destroy",
			   GTK_SIGNAL_FUNC(gtk_main_quit),
			   NULL);
       gtk_signal_connect (GTK_OBJECT (window), "delete_event",
			   GTK_SIGNAL_FUNC (gtk_false),
			   NULL);
       
       vbox1 = gtk_vbox_new (FALSE, 0);
       gtk_container_add (GTK_CONTAINER (window), vbox1);
       gtk_widget_show (vbox1);
       
       hbox1 = gtk_hbox_new (FALSE, 10);
       gtk_container_border_width (GTK_CONTAINER (hbox1), 10);
       gtk_box_pack_start (GTK_BOX (vbox1), hbox1, TRUE, TRUE, 0);
       gtk_widget_show (hbox1);
       
       separator = gtk_hseparator_new ();
       gtk_box_pack_start (GTK_BOX (vbox1), separator, FALSE, TRUE, 0);
       gtk_widget_show (separator);

       
       vbox2 = gtk_vbox_new (FALSE, 0);
       gtk_container_border_width (GTK_CONTAINER (vbox2), 10);
       gtk_box_pack_start (GTK_BOX (hbox1), vbox2, TRUE, TRUE, 0);
       gtk_widget_show (vbox2);
       
       
       /* timer stuff goes in here */
       frame1 = gtk_frame_new ("Sleep period\n(mins)");
       gtk_container_border_width (GTK_CONTAINER (frame1), 10);
       gtk_box_pack_start (GTK_BOX (vbox2), frame1, FALSE, FALSE, 0);
       gtk_widget_show (frame1);
              
       entry = GTK_ENTRY (gtk_entry_new ());
       gtk_entry_set_text (GTK_ENTRY (entry), "15");
       gtk_editable_select_region (GTK_EDITABLE (entry), 0, -1);
       gtk_container_add (GTK_CONTAINER (frame1), GTK_WIDGET (entry));
       gtk_widget_show (GTK_WIDGET (entry));
       
       /* nice little pixmap goes in here */
       
       style = gtk_widget_get_style (vbox2);
       pixmap = gdk_pixmap_create_from_xpm_d (window->window, &mask,
					    &style->bg[GTK_STATE_NORMAL],
					    Pictorial_xpm);
       pixmapwid = gtk_pixmap_new (pixmap, mask);
       gtk_box_pack_start (GTK_BOX (vbox2), pixmapwid, TRUE, TRUE, 0);
       gtk_widget_show (pixmapwid);
       
       /* some text about me */
       label = gtk_label_new ("Wallpaper Changer
	wierzbr@mcmaster.ca
	http://mcss.mcmaster.ca/~wierzbr
	WallP 0.55 ALPHA");
       gtk_misc_set_alignment (GTK_MISC(label), 0, 0.5);
       gtk_box_pack_start (GTK_BOX (vbox2), label, TRUE, TRUE, 0);
       gtk_widget_show (label);
	
       vbox3 = gtk_vbox_new (FALSE, 10);
       gtk_container_border_width (GTK_CONTAINER (vbox3), 10);
       gtk_box_pack_start (GTK_BOX(hbox1), vbox3, TRUE, TRUE, 0);
       gtk_widget_show (vbox3);
       
       /* scrolled list goes in here */
       
       frame2 = gtk_frame_new ("Images");
       gtk_container_border_width (GTK_CONTAINER (frame1), 10);
       gtk_box_pack_start (GTK_BOX (vbox3), frame2, FALSE, FALSE, 0);
       gtk_widget_show (frame2);

       scrolled_win = gtk_scrolled_window_new (NULL, NULL);
       gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_win),
				       GTK_POLICY_AUTOMATIC,
				       GTK_POLICY_AUTOMATIC);
       gtk_widget_set_usize (scrolled_win, 250, 100);
       gtk_container_add (GTK_CONTAINER (frame2), scrolled_win);
       gtk_widget_show (scrolled_win);

	list = gtk_list_new ();
	gtk_list_set_selection_mode (GTK_LIST (list), GTK_SELECTION_MULTIPLE);
	gtk_list_set_selection_mode (GTK_LIST (list), GTK_SELECTION_BROWSE);
	gtk_container_add (GTK_CONTAINER (scrolled_win), list);
	gtk_widget_show (list);
       
	button = gtk_button_new_with_label ("add");
	GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
			    GTK_SIGNAL_FUNC(file_selection),
			    NULL);
	gtk_box_pack_start (GTK_BOX (vbox3), button, FALSE, TRUE, 0);
	gtk_widget_show (button);

	button = gtk_button_new_with_label ("remove");
	GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
			    GTK_SIGNAL_FUNC(list_remove),
			    list);

	gtk_box_pack_start (GTK_BOX (vbox3), button, FALSE, TRUE, 0);
	gtk_widget_show (button);
         
       /* close ok buttons in here */
       hbox2 = gtk_hbox_new (FALSE, 10);
       gtk_container_border_width (GTK_CONTAINER (hbox2), 10);
       gtk_box_pack_start (GTK_BOX (vbox1), hbox2, FALSE, TRUE, 0);
       gtk_widget_show (hbox2);
       
       /*Close button - End program */
       button = gtk_button_new_with_label ("quit");
       gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
				  GTK_SIGNAL_FUNC(gtk_widget_destroy),
				  GTK_OBJECT (window));
       gtk_box_pack_start (GTK_BOX (hbox2), button, TRUE, TRUE, 0);
       GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
       gtk_widget_grab_default (button);
       gtk_widget_show (button);

    	/*Save button - Save list to file */
       button = gtk_button_new_with_label ("save");
       GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
       gtk_signal_connect (GTK_OBJECT (button), "clicked",
			    GTK_SIGNAL_FUNC(dump_list_to_file),
			    NULL);
	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
       gtk_box_pack_start (GTK_BOX (hbox2), button, TRUE, TRUE, 0);
       gtk_widget_show (button);
       
       /*Remove all button - clear the list*/
       button = gtk_button_new_with_label ("remove all");
       GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
       gtk_signal_connect (GTK_OBJECT (button), "clicked",
			    GTK_SIGNAL_FUNC(remove_all),
			    NULL);
	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
       gtk_box_pack_start (GTK_BOX (hbox2), button, TRUE, TRUE, 0);
       gtk_widget_show (button);              

       /*Stop button - stop changing the backgrounds*/
       button = gtk_button_new_with_label ("stop");
       GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
       gtk_signal_connect (GTK_OBJECT (button), "clicked",
			    GTK_SIGNAL_FUNC(stop_all),
			    NULL);
       GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
       gtk_box_pack_start (GTK_BOX (hbox2), button, TRUE, TRUE, 0);
       gtk_widget_show (button);
             
      /*Ok button - Start changing backgrounds*/
       button = gtk_button_new_with_label ("start");
       gtk_signal_connect (GTK_OBJECT (button), "clicked",
			   GTK_SIGNAL_FUNC (start_idle),
			   GTK_ENTRY (entry));
       
       gtk_box_pack_start (GTK_BOX (hbox2), button, TRUE, TRUE, 0);
       GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
       gtk_widget_grab_default (button);
       gtk_widget_show(button);
	
       /*Window manager hints*/       
       gdk_window_set_icon (window->window, NULL, pixmap, mask);
       gdk_window_set_icon_name (window->window, "WallP 0.55");
       gdk_window_set_functions (window->window, GDK_FUNC_ALL |
				 GDK_FUNC_RESIZE);
       gdk_window_set_decorations (window->window, GDK_DECOR_ALL |
				   GDK_DECOR_MENU);
       
    }

  if (!GTK_WIDGET_VISIBLE (window))
    gtk_widget_show (window);
  else
    gtk_widget_destroy (window);
}

int
main (int argc, char *argv[])
{
   int i=1;

   gtk_set_locale ();
   /* parse the command line */
   for (i=1; i<argc; i++)
     {
	/* -style switch */
	if (strcmp (argv[i], "-style")==0)
	  {
	     if (strcmp (argv[i+1], "center")==0)
	       {
		  STYLE = CENTER;
		  break;
	       }
	     else if (strcmp (argv[i+1], "tile")==0)
	       {
		  STYLE = TILE;
		  break;
	       }
	     else if (strcmp (argv[i+1], "scale")==0)
	       {
		  STYLE = SCALE;
		  break;
	       }
	     else
	       {
		  printf ("Style unknown, you can choose tile, scale, or center\n");
		  break;
	       }
	  }
	/* -start switch */
	if (strcmp (argv[i], "-start")==0)
	  AUTO_START = 1;
	/* -quit switch */
	if (strcmp (argv[i], "-quit")==0)
	  {
	     gtk_init (&argc, &argv);
	     gdk_imlib_init ();
	     put_image_on_root (argv[i+1], STYLE);
	     exit (0);
	  }
	  
	/* -help switch */
	if (strcmp (argv[i], "-help")==0)
	  {
	     printf ("Options: -start, -quit <image>, -style <scale> <center> <tile>\n");
	     exit (0);
	  }
     }

   if (!STYLE){
	printf ("You can now run WallP with the -style switch to specify the style you want. Ex: -style center, -style tile, -style scale, will center, tile or scale images\nYou didn't specify a style, defaulting to tile\n");
	STYLE = TILE;
   }
   gtk_init (&argc, &argv);
   create_list ();
   f_save_file();
   read_list_from_file ();
   gdk_imlib_init ();
   if (AUTO_START)
     {
	GtkWidget *w;
	start_idle (w, entry);
     }
   gtk_main ();
   return 0;
}

static char last_visit[64];

static void
file_ok_sel (GtkWidget *w, GtkFileSelection *fs)
{   
   sprintf (last_visit, gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
   list_add (last_visit);
   gtk_widget_destroy (GTK_WIDGET(fs));
}

static void
destroy (GtkWidget *widget, gpointer *data)
{
   gtk_main_quit ();
}

static void
add_all_files (GtkWidget *widget, GtkFileSelection *fs)
{
   DIR *dp;
   char buff[128];
   char tmp [128];
   struct dirent *dir;
   int pos;
   
   sprintf (buff,gtk_file_selection_get_filename (GTK_FILE_SELECTION(fs)));
   if (!(dp = opendir (buff))){
    pos = strlen(buff) - strlen(strrchr(buff, '/'));
    buff[pos]='\0';
    dp = opendir (buff);
    }
   
   while ((dir=readdir(dp))){
   
    if ((strcmp(".", dir->d_name)))
    	if (strcmp ("..", dir->d_name))
	    if (strcmp (".xvpics", dir->d_name)){
	    strcpy (tmp, buff);
	    if (tmp[strlen(tmp)-1] != '/')
	    	strcat (tmp, "/");
	    strcat (tmp, dir->d_name);
    	    list_add (tmp);
	}
    }	
   closedir (dp);
   gtk_widget_destroy (GTK_WIDGET(fs));
}                

static void
file_selection ()
{
   static GtkWidget *filew=NULL;
   GtkWidget *button;
   
   if (!filew)
     {
	/* Create a new file selection widget */
	filew = gtk_file_selection_new ("Select an image");
	gtk_window_position (GTK_WINDOW (filew), GTK_WIN_POS_MOUSE);
  
	gtk_signal_connect (GTK_OBJECT (filew), "destroy",
			    GTK_SIGNAL_FUNC(gtk_widget_destroyed)
			    , &filew);
   
	/* Connect the ok_button to file_ok_sel function */
	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION 
					(filew)->ok_button),
			    "clicked", GTK_SIGNAL_FUNC (file_ok_sel), 
			    filew);
         
	/* Connect the cancel_button to destroy the widget */
	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION 
					       (filew)->cancel_button),
				   "clicked",
				   GTK_SIGNAL_FUNC (gtk_widget_destroy),
				   GTK_OBJECT (filew));
         
	gtk_file_selection_set_filename (GTK_FILE_SELECTION(filew), 
					 last_visit);
    	/* Add all files button */
        button = gtk_button_new_with_label ("Add all");
        gtk_signal_connect (GTK_OBJECT (button), "clicked",
                          (GtkSignalFunc) add_all_files,
                          (gpointer) filew);
        gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (filew)->action_area),
                          button, FALSE, FALSE, 0);
        gtk_widget_show (button); 
     }
   if (!GTK_WIDGET_VISIBLE (filew))
     gtk_widget_show(filew);
   else
     gtk_widget_destroy (filew);
}

